@@ -183,7 +183,8 @@ testfile(TSK_FS_INFO * a_fs, TSK_INUM_T a_inum)
     s_off = 0;
     if (tsk_fs_file_walk(file1, (TSK_FS_FILE_WALK_FLAG_ENUM) 0,
             fw_action1, NULL)) {
-        fprintf(stderr, "Error walking file inode: %"PRIuINUM"\n", a_inum);
+        fprintf(stderr, "Error walking file inode: %" PRIuINUM "\n",
+            a_inum);
         tsk_error_print(stderr);
         tsk_error_reset();
         return 1;
@@ -202,13 +203,11 @@ test_fat12()
 {
     TSK_FS_INFO *fs;
     TSK_IMG_INFO *img;
-    char *tname = "fat12.dd";
+    const char *tname = "fat12.dd";
     char fname[512];
 
     snprintf(fname, 512, "%s/fat12.dd", s_root);
-    if ((img =
-            tsk_img_open_sing((const TSK_TCHAR *) fname,
-                (TSK_IMG_TYPE_ENUM) 0)) == NULL) {
+    if ((img = tsk_img_open_sing(fname, (TSK_IMG_TYPE_ENUM) 0, 0)) == NULL) {
         fprintf(stderr, "Error opening %s image\n", tname);
         tsk_error_print(stderr);
         tsk_error_reset();
@@ -240,16 +239,14 @@ test_fat_slack()
 {
     TSK_FS_INFO *fs;
     TSK_IMG_INFO *img;
-    char *tname = "fat-img-kw";
+    const char *tname = "fat-img-kw";
     char fname[512];
     TSK_FS_FILE *file1;
     char buf[512];
     ssize_t retval;
 
     snprintf(fname, 512, "%s/fat-img-kw.dd", s_root);
-    if ((img =
-            tsk_img_open_sing((const TSK_TCHAR *) fname,
-                (TSK_IMG_TYPE_ENUM) 0)) == NULL) {
+    if ((img = tsk_img_open_sing(fname, (TSK_IMG_TYPE_ENUM) 0, 0)) == NULL) {
         fprintf(stderr, "Error opening %s image\n", tname);
         tsk_error_print(stderr);
         tsk_error_reset();
@@ -335,7 +332,7 @@ test_fat_recover()
 {
     TSK_FS_INFO *fs;
     TSK_IMG_INFO *img;
-    char *tname = "fe_test_1.img-FAT";
+    const char *tname = "fe_test_1.img-FAT";
     char fname[512];
     TSK_FS_FILE *file1;
     TSK_FS_FILE *file2;
@@ -343,9 +340,7 @@ test_fat_recover()
     ssize_t retval;
 
     snprintf(fname, 512, "%s/fe_test_1.img", s_root);
-    if ((img =
-            tsk_img_open_sing((const TSK_TCHAR *) fname,
-                (TSK_IMG_TYPE_ENUM) 0)) == NULL) {
+    if ((img = tsk_img_open_sing(fname, (TSK_IMG_TYPE_ENUM) 0, 0)) == NULL) {
         fprintf(stderr, "Error opening %s image\n", tname);
         tsk_error_print(stderr);
         tsk_error_reset();
@@ -363,7 +358,7 @@ test_fat_recover()
 
 
     // fragmented.html
-    char *fname2 = "fragmented.html";
+    const char *fname2 = "fragmented.html";
     file1 = tsk_fs_file_open_meta(fs, NULL, 1162);
     if (file1 == NULL) {
         fprintf(stderr, "Error opening %s (%s)\n", fname2, tname);
@@ -478,16 +473,14 @@ test_ntfs_slack_ads()
 {
     TSK_FS_INFO *fs;
     TSK_IMG_INFO *img;
-    char *tname = "ntfs-img-kw";
+    const char *tname = "ntfs-img-kw";
     char fname[512];
     TSK_FS_FILE *file1;
     char buf[512];
     ssize_t retval;
 
     snprintf(fname, 512, "%s/ntfs-img-kw-1.dd", s_root);
-    if ((img =
-            tsk_img_open_sing((const TSK_TCHAR *) fname,
-                (TSK_IMG_TYPE_ENUM) 0)) == NULL) {
+    if ((img = tsk_img_open_sing(fname, (TSK_IMG_TYPE_ENUM) 0, 0)) == NULL) {
         fprintf(stderr, "Error opening %s image\n", tname);
         tsk_error_print(stderr);
         tsk_error_reset();
@@ -585,11 +578,11 @@ test_ntfs_slack_ads()
     }
 
     // test the getsize API for both attributes
-    const TSK_FS_ATTR *fs_attr = tsk_fs_file_attr_get_type(file1, TSK_FS_ATTR_TYPE_NTFS_DATA, 3, 1);
+    const TSK_FS_ATTR *fs_attr =
+        tsk_fs_file_attr_get_type(file1, TSK_FS_ATTR_TYPE_NTFS_DATA, 3, 1);
     if (!fs_attr) {
         fprintf(stderr,
-            "Error getting data attribute 3 in file-n-5.dat (%s)",
-            tname);
+            "Error getting data attribute 3 in file-n-5.dat (%s)", tname);
         tsk_error_print(stderr);
         tsk_error_reset();
         return 1;
@@ -601,7 +594,8 @@ test_ntfs_slack_ads()
         return 1;
     }
 
-    fs_attr = tsk_fs_file_attr_get_type(file1, TSK_FS_ATTR_TYPE_NTFS_DATA, 5, 1);
+    fs_attr =
+        tsk_fs_file_attr_get_type(file1, TSK_FS_ATTR_TYPE_NTFS_DATA, 5, 1);
     if (!fs_attr) {
         fprintf(stderr,
             "Error getting size of attribute 5 in file-n-5.dat (%s)",
@@ -629,13 +623,11 @@ test_ntfs_fe()
 {
     TSK_FS_INFO *fs;
     TSK_IMG_INFO *img;
-    char *tname = "fe_test_1-NTFS";
+    const char *tname = "fe_test_1-NTFS";
     char fname[512];
 
     snprintf(fname, 512, "%s/fe_test_1.img", s_root);
-    if ((img =
-            tsk_img_open_sing((const TSK_TCHAR *) fname,
-                (TSK_IMG_TYPE_ENUM) 0)) == NULL) {
+    if ((img = tsk_img_open_sing(fname, (TSK_IMG_TYPE_ENUM) 0, 0)) == NULL) {
         fprintf(stderr, "Error opening %s image\n", tname);
         tsk_error_print(stderr);
         tsk_error_reset();
@@ -669,13 +661,11 @@ test_ntfs_comp()
 {
     TSK_FS_INFO *fs;
     TSK_IMG_INFO *img;
-    char *tname = "ntfs-comp-1";
+    const char *tname = "ntfs-comp-1";
     char fname[512];
 
     snprintf(fname, 512, "%s/ntfs-comp-1.img", s_root);
-    if ((img =
-            tsk_img_open_sing((const TSK_TCHAR *) fname,
-                (TSK_IMG_TYPE_ENUM) 0)) == NULL) {
+    if ((img = tsk_img_open_sing(fname, (TSK_IMG_TYPE_ENUM) 0, 0)) == NULL) {
         fprintf(stderr, "Error opening %s image\n", tname);
         tsk_error_print(stderr);
         tsk_error_reset();
