@@ -24,15 +24,17 @@ usage()
 {
     TFPRINTF(stderr,
         _TSK_T
-        ("%s [-i imgtype] [-o imgoffset] [-brvV] [-aAmM] [-t vstype] image [images]\n"),
+        ("%s [-i imgtype] [-b dev_sector_size] [-o imgoffset] [-BrvV] [-aAmM] [-t vstype] image [images]\n"),
         progname);
     tsk_fprintf(stderr,
         "\t-t vstype: The type of volume system (use '-t list' for list of supported types)\n");
     tsk_fprintf(stderr,
         "\t-i imgtype: The format of the image file (use '-i list' for list supported types)\n");
+    tsk_fprintf(stderr,
+        "\t-b dev_sector_size: The size (in bytes) of the device sectors\n");
     tsk_fprintf(stderr,
         "\t-o imgoffset: Offset to the start of the volume that contains the partition system (in sectors)\n");
-    tsk_fprintf(stderr, "\t-b: print the rounded length in bytes\n");
+    tsk_fprintf(stderr, "\t-B: print the rounded length in bytes\n");
     tsk_fprintf(stderr,
         "\t-r: recurse and look for other partition tables in partitions (DOS Only)\n");
     tsk_fprintf(stderr, "\t-v: verbose output\n");
@@ -78,26 +80,26 @@ part_act(TSK_VS_INFO * vs, const TSK_VS_PART_INFO * part, void *ptr)
     if (print_bytes) {
         TSK_OFF_T size;
         char unit = ' ';
-        size = part->len;
+        size = part->len * part->vs->block_size;
 
-        if (part->len < 2) {
-            size = 512 * part->len;
+        if (part->len < 1024) {
             unit = 'B';
         }
-        else if (size < (2 << 10)) {
-            size = part->len / 2;
+        else if (size < 512*1024) {
+            size = part->len / 1024;
             unit = 'K';
         }
-        else if (size < (2 << 20)) {
-            size = part->len / (2 << 10);
+        else if (size < (512*1048576)) {
+            size = part->len / 1048576;
             unit = 'M';
         }
-        else if (size < ((TSK_OFF_T) 2 << 30)) {
-            size = part->len / (2 << 20);
+        else if (size < (512*(TSK_OFF_T)1073741824)) {
+            size = part->len / ((TSK_OFF_T)1073741824);
             unit = 'G';
         }
-        else if (size < ((TSK_OFF_T) 2 << 40)) {
-            size = part->len / (2 << 30);
+        // 1099511627776
+        else if (size < (512*((TSK_OFF_T)1<<40))) {
+            size = part->len / ((TSK_OFF_T)1<<40);
             unit = 'T';
         }
 
@@ -117,9 +119,8 @@ part_act(TSK_VS_INFO * vs, const TSK_VS_PART_INFO * part, void *ptr)
 
     if ((recurse) && (vs->vstype == TSK_VS_TYPE_DOS)
         && (part->flags == TSK_VS_PART_FLAG_ALLOC)) {
-        // @@@ This assumes 512-byte sectors
         if (recurse_cnt < 64)
-            recurse_list[recurse_cnt++] = part->start * 512;
+            recurse_list[recurse_cnt++] = part->start * part->vs->block_size;
     }
 
     return TSK_WALK_CONT;
@@ -142,33 +143,35 @@ print_header(const TSK_VS_INFO * vs)
 
 
 int
-main(int argc, char ** argv1)
+main(int argc, char **argv1)
 {
     TSK_VS_INFO *vs;
     int ch;
-    TSK_OFF_T imgoff = 0;
+    TSK_OFF_T imgaddr = 0;
     int flags = 0;
     TSK_IMG_INFO *img;
     TSK_IMG_TYPE_ENUM imgtype = TSK_IMG_TYPE_DETECT;
     TSK_VS_TYPE_ENUM vstype = TSK_VS_TYPE_DETECT;
     uint8_t hide_meta = 0;
     TSK_TCHAR **argv;
-    
+    unsigned int ssize = 0;
+    TSK_TCHAR *cp;
+
 #ifdef TSK_WIN32
     // On Windows, get the wide arguments (mingw doesn't support wmain)
     argv = CommandLineToArgvW(GetCommandLineW(), &argc);
-    if( argv == NULL) {    
+    if (argv == NULL) {
         fprintf(stderr, "Error getting wide arguments\n");
         exit(1);
     }
 #else
-    argv = (TSK_TCHAR **)argv1;
+    argv = (TSK_TCHAR **) argv1;
 #endif
-    
+
 
     progname = argv[0];
 
-    while ((ch = GETOPT(argc, argv, _TSK_T("aAbi:mMo:rt:vV"))) > 0) {
+    while ((ch = GETOPT(argc, argv, _TSK_T("aAb:Bi:mMo:rt:vV"))) > 0) {
         switch (ch) {
         case _TSK_T('a'):
             flags |= TSK_VS_PART_FLAG_ALLOC;
@@ -176,9 +179,19 @@ main(int argc, char ** argv1)
         case _TSK_T('A'):
             flags |= TSK_VS_PART_FLAG_UNALLOC;
             break;
-        case _TSK_T('b'):
+        case _TSK_T('B'):
             print_bytes = 1;
             break;
+        case _TSK_T('b'):
+            ssize = (unsigned int) TSTRTOUL(OPTARG, &cp, 0);
+            if (*cp || *cp == *OPTARG || ssize < 1) {
+                TFPRINTF(stderr,
+                    _TSK_T
+                    ("invalid argument: sector size must be positive: %s\n"),
+                    OPTARG);
+                usage();
+            }
+            break;
         case _TSK_T('i'):
             if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
                 tsk_img_type_print(stderr);
@@ -199,7 +212,7 @@ main(int argc, char ** argv1)
             hide_meta = 1;
             break;
         case _TSK_T('o'):
-            if ((imgoff = tsk_parse_offset(OPTARG)) == -1) {
+            if ((imgaddr = tsk_parse_offset(OPTARG)) == -1) {
                 tsk_error_print(stderr);
                 exit(1);
             }
@@ -251,23 +264,21 @@ main(int argc, char ** argv1)
     }
 
     /* open the image */
-    img =
-        tsk_img_open(argc - OPTIND,
-        (const TSK_TCHAR **) &argv[OPTIND], imgtype);
+    img = tsk_img_open(argc - OPTIND, &argv[OPTIND], imgtype, ssize);
 
     if (img == NULL) {
         tsk_error_print(stderr);
         exit(1);
     }
-    if (imgoff >= img->size) {
+    if ((imgaddr * img->sector_size) >= img->size) {
         tsk_fprintf(stderr,
             "Sector offset supplied is larger than disk image (maximum: %"
-            PRIu64 ")\n", img->size / 512);
+            PRIu64 ")\n", img->size / img->sector_size);
         exit(1);
     }
 
     /* process the partition tables */
-    vs = tsk_vs_open(img, (TSK_OFF_T) imgoff, vstype);
+    vs = tsk_vs_open(img, imgaddr * img->sector_size, vstype);
     if (vs == NULL) {
         tsk_error_print(stderr);
         if (tsk_errno == TSK_ERR_VS_UNSUPTYPE)
