@@ -95,14 +95,12 @@ fatfs_dir_buf_add(FATFS_INFO * fatfs, TSK_INUM_T par_inum,
     if (fatfs->dir_buf_next == fatfs->dir_buf_size) {
         fatfs->dir_buf_size += 256;
         if ((fatfs->dir_buf =
-                (TSK_INUM_T *) tsk_realloc(fatfs->
-                    dir_buf,
+                (TSK_INUM_T *) tsk_realloc(fatfs->dir_buf,
                     fatfs->dir_buf_size * sizeof(TSK_INUM_T))) == NULL) {
             return 1;
         }
         if ((fatfs->par_buf =
-                (TSK_INUM_T *) tsk_realloc(fatfs->
-                    par_buf,
+                (TSK_INUM_T *) tsk_realloc(fatfs->par_buf,
                     fatfs->dir_buf_size * sizeof(TSK_INUM_T))) == NULL) {
             return 1;
         }
@@ -195,8 +193,13 @@ fatfs_dent_parse_buf(FATFS_INFO * fatfs, TSK_FS_DIR * a_fs_dir, char *buf,
             int i;
 
             /* is it a valid dentry? */
-            if (0 == fatfs_isdentry(fatfs, dep))
+            if (0 == fatfs_isdentry(fatfs, dep)) {
+                if (tsk_verbose)
+                    tsk_fprintf(stderr,
+                        "fatfs_dent_parse_buf: Entry %ui is invalid\n",
+                        idx);
                 continue;
+            }
 
             /* Copy the directory entry into the TSK_FS_NAME structure */
             dir = (fatfs_dentry *) dep;
@@ -291,8 +294,8 @@ fatfs_dent_parse_buf(FATFS_INFO * fatfs, TSK_FS_DIR * a_fs_dir, char *buf,
 
                     /* Convert the UTF16 to UTF8 */
                     UTF16 *name16 =
-                        (UTF16 *) ((uintptr_t) & lfninfo.
-                        name[lfninfo.start + 1]);
+                        (UTF16 *) ((uintptr_t) & lfninfo.name[lfninfo.
+                            start + 1]);
                     UTF8 *name8 = (UTF8 *) fs_name->name;
 
                     retVal =
@@ -422,6 +425,14 @@ fatfs_dent_parse_buf(FATFS_INFO * fatfs, TSK_FS_DIR * a_fs_dir, char *buf,
                          * then we do not care about the value of '..' and this can only cause
                          * infinite loop problems */
                         inode = fs_name->meta_addr = 0;
+                        dir_found = 1;
+                    }
+                    if ((dir_found == 0)
+                        && (addrs[0] == fatfs->firstdatasect)) {
+                        /* if we are currently in the root directory, we aren't going to find
+                         * a parent.  This shouldn't happen, but could result in an infinite loop. */
+                        inode = fs_name->meta_addr = 0;
+                        dir_found = 1;
                     }
                     if (dir_found == 0) {
                         /* The parent directory is not in the list.  We are going to walk
@@ -646,6 +657,13 @@ fatfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
             "fatfs_dir_open_meta: Processing directory %" PRIuINUM "\n",
             a_addr);
 
+    if (size == 0) {
+        if (tsk_verbose)
+            tsk_fprintf(stderr,
+                "fatfs_dir_open_meta: directory has 0 size\n");
+        return TSK_OK;
+    }
+
     /* Make a copy of the directory contents using file_walk */
     if ((dirbuf = tsk_malloc((size_t) len)) == NULL) {
         return TSK_ERR;
@@ -748,3 +766,9 @@ fatfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
 
     return retval;
 }
+
+int
+fatfs_name_cmp(TSK_FS_INFO * a_fs_info, const char *s1, const char *s2)
+{
+    return strcasecmp(s1, s2);
+}
