@@ -95,7 +95,7 @@ is_83_name(fatfs_dentry * de)
 ** 
 */
 static time_t
-dos2unixtime(uint16_t date, uint16_t time)
+dos2unixtime(uint16_t date, uint16_t time, uint8_t timetens)
 {
     struct tm tm1;
     time_t ret;
@@ -108,6 +108,9 @@ dos2unixtime(uint16_t date, uint16_t time)
     tm1.tm_sec = ((time & FATFS_SEC_MASK) >> FATFS_SEC_SHIFT) * 2;
     if ((tm1.tm_sec < 0) || (tm1.tm_sec > 60))
         tm1.tm_sec = 0;
+    // the ctimetens value has a range of 0 to 199
+    if (timetens > 100)
+        tm1.tm_sec++;
 
     tm1.tm_min = ((time & FATFS_MIN_MASK) >> FATFS_MIN_SHIFT);
     if ((tm1.tm_min < 0) || (tm1.tm_min > 59))
@@ -155,6 +158,13 @@ dos2unixtime(uint16_t date, uint16_t time)
     return ret;
 }
 
+/* timetens is number of tenths of a second for a 2 second range (values 0 to 199) */
+static uint32_t
+dos2nanosec(uint8_t timetens)
+{
+    timetens %= 100;
+    return timetens * 10000000;
+}
 
 
 /* 
@@ -263,6 +273,8 @@ fatfs_dinode_copy(FATFS_INFO * fatfs, TSK_FS_META * fs_meta,
         fs_meta->atime = 0;
         fs_meta->ctime = 0;
         fs_meta->crtime = 0;
+        fs_meta->mtime_nano = fs_meta->atime_nano = fs_meta->ctime_nano =
+            fs_meta->crtime_nano = 0;
     }
     else {
         /* There is no notion of link in FAT, just deleted or not */
@@ -273,30 +285,37 @@ fatfs_dinode_copy(FATFS_INFO * fatfs, TSK_FS_META * fs_meta,
         if (FATFS_ISDATE(tsk_getu16(fs->endian, in->wdate)))
             fs_meta->mtime =
                 dos2unixtime(tsk_getu16(fs->endian, in->wdate),
-                tsk_getu16(fs->endian, in->wtime));
+                tsk_getu16(fs->endian, in->wtime), 0);
         else
             fs_meta->mtime = 0;
+        fs_meta->mtime_nano = 0;
 
         if (FATFS_ISDATE(tsk_getu16(fs->endian, in->adate)))
             fs_meta->atime =
-                dos2unixtime(tsk_getu16(fs->endian, in->adate), 0);
+                dos2unixtime(tsk_getu16(fs->endian, in->adate), 0, 0);
         else
             fs_meta->atime = 0;
+        fs_meta->atime_nano = 0;
 
 
         /* cdate is the creation date in FAT and there is no change,
          * so we just put in into change and set create to 0.  The other
          * front-end code knows how to handle it and display it
          */
-        if (FATFS_ISDATE(tsk_getu16(fs->endian, in->cdate)))
+        if (FATFS_ISDATE(tsk_getu16(fs->endian, in->cdate))) {
             fs_meta->crtime =
                 dos2unixtime(tsk_getu16(fs->endian, in->cdate),
-                tsk_getu16(fs->endian, in->ctime));
-        else
+                tsk_getu16(fs->endian, in->ctime), in->ctimeten);
+            fs_meta->crtime_nano = dos2nanosec(in->ctimeten);
+        }
+        else {
             fs_meta->crtime = 0;
+            fs_meta->crtime_nano = 0;
+        }
 
         // FAT does not have a changed time
         fs_meta->ctime = 0;
+        fs_meta->ctime_nano = 0;
     }
 
     /* Values that do not exist in FAT */
@@ -346,9 +365,8 @@ fatfs_dinode_copy(FATFS_INFO * fatfs, TSK_FS_META * fs_meta,
         retVal = tsk_UTF16toUTF8(fs->endian, (const UTF16 **) &name16,
             (UTF16 *) & lfn->part2[12],
             &name8,
-            (UTF8 *) ((uintptr_t) fs_meta->name2->
-                name + sizeof(fs_meta->name2->name)),
-            TSKlenientConversion);
+            (UTF8 *) ((uintptr_t) fs_meta->name2->name +
+                sizeof(fs_meta->name2->name)), TSKlenientConversion);
 
         if (retVal != TSKconversionOK) {
             tsk_error_reset();
@@ -366,9 +384,8 @@ fatfs_dinode_copy(FATFS_INFO * fatfs, TSK_FS_META * fs_meta,
         retVal = tsk_UTF16toUTF8(fs->endian, (const UTF16 **) &name16,
             (UTF16 *) & lfn->part3[4],
             &name8,
-            (UTF8 *) ((uintptr_t) fs_meta->name2->
-                name + sizeof(fs_meta->name2->name)),
-            TSKlenientConversion);
+            (UTF8 *) ((uintptr_t) fs_meta->name2->name +
+                sizeof(fs_meta->name2->name)), TSKlenientConversion);
 
         if (retVal != TSKconversionOK) {
             tsk_error_reset();
@@ -539,6 +556,8 @@ fatfs_make_root(FATFS_INFO * fatfs, TSK_FS_META * fs_meta)
     fs_meta->flags = (TSK_FS_META_FLAG_USED | TSK_FS_META_FLAG_ALLOC);
     fs_meta->uid = fs_meta->gid = 0;
     fs_meta->mtime = fs_meta->atime = fs_meta->ctime = fs_meta->crtime = 0;
+    fs_meta->mtime_nano = fs_meta->atime_nano = fs_meta->ctime_nano =
+        fs_meta->crtime_nano = 0;
 
     if (fs_meta->name2 == NULL) {
         if ((fs_meta->name2 = (TSK_FS_META_NAME_LIST *)
@@ -548,6 +567,10 @@ fatfs_make_root(FATFS_INFO * fatfs, TSK_FS_META * fs_meta)
     }
     fs_meta->name2->name[0] = '\0';
 
+    fs_meta->attr_state = TSK_FS_META_ATTR_EMPTY;
+    if (fs_meta->attr) {
+        tsk_fs_attrlist_markunused(fs_meta->attr);
+    }
     addr_ptr = (TSK_DADDR_T *) fs_meta->content_ptr;
 
     /* TSK_FS_TYPE_FAT12 and TSK_FS_TYPE_FAT16 don't use the FAT for root directory, so 
@@ -628,6 +651,8 @@ fatfs_make_mbr(FATFS_INFO * fatfs, TSK_FS_META * fs_meta)
     fs_meta->flags = (TSK_FS_META_FLAG_USED | TSK_FS_META_FLAG_ALLOC);
     fs_meta->uid = fs_meta->gid = 0;
     fs_meta->mtime = fs_meta->atime = fs_meta->ctime = fs_meta->crtime = 0;
+    fs_meta->mtime_nano = fs_meta->atime_nano = fs_meta->ctime_nano =
+        fs_meta->crtime_nano = 0;
 
     if (fs_meta->name2 == NULL) {
         if ((fs_meta->name2 = (TSK_FS_META_NAME_LIST *)
@@ -638,6 +663,11 @@ fatfs_make_mbr(FATFS_INFO * fatfs, TSK_FS_META * fs_meta)
     strncpy(fs_meta->name2->name, FATFS_MBRNAME,
         TSK_FS_META_NAME_LIST_NSIZE);
 
+    fs_meta->attr_state = TSK_FS_META_ATTR_EMPTY;
+    if (fs_meta->attr) {
+        tsk_fs_attrlist_markunused(fs_meta->attr);
+    }
+
     addr_ptr = (TSK_DADDR_T *) fs_meta->content_ptr;
     addr_ptr[0] = 0;
     fs_meta->size = 512;
@@ -667,6 +697,8 @@ fatfs_make_fat(FATFS_INFO * fatfs, uint8_t a_which, TSK_FS_META * fs_meta)
     fs_meta->flags = (TSK_FS_META_FLAG_USED | TSK_FS_META_FLAG_ALLOC);
     fs_meta->uid = fs_meta->gid = 0;
     fs_meta->mtime = fs_meta->atime = fs_meta->ctime = fs_meta->crtime = 0;
+    fs_meta->mtime_nano = fs_meta->atime_nano = fs_meta->ctime_nano =
+        fs_meta->crtime_nano = 0;
 
     if (fs_meta->name2 == NULL) {
         if ((fs_meta->name2 = (TSK_FS_META_NAME_LIST *)
@@ -690,8 +722,14 @@ fatfs_make_fat(FATFS_INFO * fatfs, uint8_t a_which, TSK_FS_META * fs_meta)
         addr_ptr[0] = fatfs->firstfatsect + fatfs->sectperfat;
     }
     else {
-        ////XXXX 
+        ////@@@ 
     }
+
+    fs_meta->attr_state = TSK_FS_META_ATTR_EMPTY;
+    if (fs_meta->attr) {
+        tsk_fs_attrlist_markunused(fs_meta->attr);
+    }
+
     fs_meta->size = fatfs->sectperfat * fs->block_size;
 
     return 0;
@@ -753,10 +791,13 @@ fatfs_isdentry(FATFS_INFO * fatfs, fatfs_dentry * de)
         else if ((tsk_getu16(fs->endian, de->cdate) != 0) &&
             (FATFS_ISDATE(tsk_getu16(fs->endian, de->cdate)) == 0))
             return 0;
+        else if (de->ctimeten > 200)
+            return 0;
         else if ((tsk_getu16(fs->endian, de->adate) != 0) &&
             (FATFS_ISDATE(tsk_getu16(fs->endian, de->adate)) == 0))
             return 0;
-        else if (FATFS_ISDATE(tsk_getu16(fs->endian, de->wdate)) == 0)
+        else if ((tsk_getu16(fs->endian, de->wdate) != 0) &&
+            (FATFS_ISDATE(tsk_getu16(fs->endian, de->wdate)) == 0))
             return 0;
 
         /* verify the starting cluster is small enough */
@@ -1283,6 +1324,8 @@ fatfs_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start_inum,
             inum <= end_inum; inum++) {
             int retval;
 
+            tsk_fs_meta_reset(fs_file->meta);
+
             if (inum == FATFS_MBRINO(fs)) {
                 if (fatfs_make_mbr(fatfs, fs_file->meta)) {
                     tsk_fs_file_close(fs_file);
@@ -1496,7 +1539,7 @@ fatfs_make_data_run(TSK_FS_FILE * a_fs_file)
     fatfs = (FATFS_INFO *) fs;
 
     clust = ((TSK_DADDR_T *) fs_meta->content_ptr)[0];
-    size_remain = fs_meta->size;
+    size_remain = roundup(fs_meta->size, fatfs->csize * fs->block_size);
 
     // see if we have already loaded the runs
     if ((fs_meta->attr != NULL)
@@ -1559,6 +1602,7 @@ fatfs_make_data_run(TSK_FS_FILE * a_fs_file)
         if (tsk_fs_attr_set_run(a_fs_file, fs_attr, data_run, NULL,
                 TSK_FS_ATTR_TYPE_DEFAULT, TSK_FS_ATTR_ID_DEFAULT,
                 data_run->len * fs->block_size,
+                data_run->len * fs->block_size,
                 data_run->len * fs->block_size, 0, 0)) {
             return 1;
         }
@@ -1595,6 +1639,7 @@ fatfs_make_data_run(TSK_FS_FILE * a_fs_file)
         if (tsk_fs_attr_set_run(a_fs_file, fs_attr, data_run, NULL,
                 TSK_FS_ATTR_TYPE_DEFAULT, TSK_FS_ATTR_ID_DEFAULT,
                 data_run->len * fs->block_size,
+                data_run->len * fs->block_size,
                 data_run->len * fs->block_size, 0, 0)) {
             return 1;
         }
@@ -1734,8 +1779,8 @@ fatfs_make_data_run(TSK_FS_FILE * a_fs_file)
             // initialize the data run
             if (tsk_fs_attr_set_run(a_fs_file, fs_attr, data_run_head,
                     NULL, TSK_FS_ATTR_TYPE_DEFAULT, TSK_FS_ATTR_ID_DEFAULT,
-                    fs_meta->size, roundup(fs_meta->size,
-                        fs->block_size), 0, 0)) {
+                    fs_meta->size, fs_meta->size, roundup(fs_meta->size,
+                        fatfs->csize * fs->block_size), 0, 0)) {
                 fs_meta->attr_state = TSK_FS_META_ATTR_ERROR;
                 return 1;
             }
@@ -1755,8 +1800,8 @@ fatfs_make_data_run(TSK_FS_FILE * a_fs_file)
             // initialize the data run
             if (tsk_fs_attr_set_run(a_fs_file, fs_attr, data_run_tmp, NULL,
                     TSK_FS_ATTR_TYPE_DEFAULT, TSK_FS_ATTR_ID_DEFAULT,
-                    fs_meta->size, roundup(fs_meta->size,
-                        fs->block_size), 0, 0)) {
+                    fs_meta->size, fs_meta->size, roundup(fs_meta->size,
+                        fatfs->csize * fs->block_size), 0, 0)) {
                 fs_meta->attr_state = TSK_FS_META_ATTR_ERROR;
                 return 1;
             }
@@ -1869,8 +1914,8 @@ fatfs_make_data_run(TSK_FS_FILE * a_fs_file)
         // initialize the data run
         if (tsk_fs_attr_set_run(a_fs_file, fs_attr, data_run_head, NULL,
                 TSK_FS_ATTR_TYPE_DEFAULT, TSK_FS_ATTR_ID_DEFAULT,
-                fs_meta->size, roundup(fs_meta->size, fs->block_size), 0,
-                0)) {
+                fs_meta->size, fs_meta->size, roundup(fs_meta->size,
+                    fatfs->csize * fs->block_size), 0, 0)) {
             fs_meta->attr_state = TSK_FS_META_ATTR_ERROR;
             return 1;
         }
