@@ -261,8 +261,11 @@ tsk_fs_attr_set_str(TSK_FS_FILE * a_fs_file, TSK_FS_ATTR * a_fs_attr,
  * @param name Name of the attribute (can be NULL)
  * @param type Type of attribute to add run to
  * @param id Id of attribute to add run to
- * @param size Total size of the attribute (can be larger than length of initial run being added)
+ * @param size Total size of the attribute (can be larger than length of initial run being added) 
+ * @param init_size Number of bytes in attribute that have been initialized (less then or equal to size)
+ * (note that this sets the initialized size for the attribute and it will not be updated as more runs are added).
  * @param alloc_size Allocated size of the attribute (>= size).  Identifies the slack space. 
+ * (note that this sets the allocated size for the attribute and it will not be updated as more runs are added).
  * @param flags Flags about compression, sparse etc. of data
  * @param compsize Compression unit size (in case it needs to be created)
  *
@@ -272,7 +275,8 @@ uint8_t
 tsk_fs_attr_set_run(TSK_FS_FILE * a_fs_file, TSK_FS_ATTR * a_fs_attr,
     TSK_FS_ATTR_RUN * a_data_run_new, const char *name,
     TSK_FS_ATTR_TYPE_ENUM type, uint16_t id, TSK_OFF_T size,
-    TSK_OFF_T alloc_size, TSK_FS_ATTR_FLAG_ENUM flags, uint32_t compsize)
+    TSK_OFF_T init_size, TSK_OFF_T alloc_size,
+    TSK_FS_ATTR_FLAG_ENUM flags, uint32_t compsize)
 {
     TSK_FS_INFO *fs;
 
@@ -308,7 +312,7 @@ tsk_fs_attr_set_run(TSK_FS_FILE * a_fs_file, TSK_FS_ATTR * a_fs_attr,
     a_fs_attr->id = id;
     a_fs_attr->size = size;
     a_fs_attr->nrd.allocsize = alloc_size;
-    a_fs_attr->nrd.initsize = 0;
+    a_fs_attr->nrd.initsize = init_size;
     a_fs_attr->nrd.compsize = compsize;
 
     if (fs_attr_put_name(a_fs_attr, name)) {
@@ -349,14 +353,10 @@ tsk_fs_attr_set_run(TSK_FS_FILE * a_fs_file, TSK_FS_ATTR * a_fs_attr,
 
     a_fs_attr->nrd.run = a_data_run_new;
 
-    // update the pointer to the end of the list and the initsize
+    // update the pointer to the end of the list
     a_fs_attr->nrd.run_end = a_data_run_new;
-    a_fs_attr->nrd.initsize =
-        (a_fs_attr->nrd.run_end->len * fs->block_size);
     while (a_fs_attr->nrd.run_end->next) {
         a_fs_attr->nrd.run_end = a_fs_attr->nrd.run_end->next;
-        a_fs_attr->nrd.initsize +=
-            (a_fs_attr->nrd.run_end->len * fs->block_size);
     }
 
     return 0;
@@ -387,7 +387,7 @@ tsk_fs_attr_add_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
         tsk_error_reset();
         tsk_errno = TSK_ERR_FS_ARG;
         snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "Null fs_attr in tsk_fs_attr_add_run");
+            "tsk_fs_attr_add_run: Error, a_fs_attr is NULL");
         return 1;
     }
 
@@ -396,7 +396,8 @@ tsk_fs_attr_add_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
         tsk_error_reset();
         tsk_errno = TSK_ERR_FS_ARG;
         snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "tsk_fs_attr_put_run: Error, NULL run added to existing attribute");
+            "tsk_fs_attr_add_run: Error, a_data_run_new is NULL (%"
+            PRIuINUM ")", a_fs_attr->fs_file->meta->addr);
         return 1;
     }
 
@@ -413,8 +414,6 @@ tsk_fs_attr_add_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
             a_data_run_new->offset)) {
 
         a_fs_attr->nrd.run_end->next = a_data_run_new;
-        a_fs_attr->nrd.initsize += (run_len * a_fs->block_size);
-
         // update the pointer to the end of the list
         while (a_fs_attr->nrd.run_end->next)
             a_fs_attr->nrd.run_end = a_fs_attr->nrd.run_end->next;
@@ -423,10 +422,18 @@ tsk_fs_attr_add_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
         return 0;
     }
 
+    // cycle through existing runs and see if we can add this into a filler spot
     data_run_cur = a_fs_attr->nrd.run;
     data_run_prev = NULL;
     while (data_run_cur) {
 
+        if (tsk_verbose)
+            tsk_fprintf(stderr,
+                "tsk_fs_attr_add: %" PRIuOFF "@%" PRIuOFF
+                " (Filler: %s)\n", data_run_cur->offset, data_run_cur->len,
+                (data_run_cur->flags & TSK_FS_ATTR_RUN_FLAG_FILLER) ? "Yes"
+                : "No");
+
         /* Do we replace this filler spot? */
         if (data_run_cur->flags & TSK_FS_ATTR_RUN_FLAG_FILLER) {
 
@@ -434,9 +441,12 @@ tsk_fs_attr_add_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
              * the filler to start from VCN 0 */
             if (data_run_cur->offset > a_data_run_new->offset) {
                 tsk_error_reset();
-                tsk_errno = TSK_ERR_FS_ARG;
+                tsk_errno = TSK_ERR_FS_GENFS;
                 snprintf(tsk_errstr, TSK_ERRSTR_L,
-                    "tsk_fs_attr_put_run: could not add data_run");
+                    "tsk_fs_attr_add_run: could not add data_run b.c. offset (%"
+                    PRIuOFF ") is larger than FILLER (%" PRIuOFF ") (%"
+                    PRIuINUM ")", a_data_run_new->offset,
+                    data_run_cur->offset, a_fs_attr->fs_file->meta->addr);
                 return 1;
             }
 
@@ -497,6 +507,8 @@ tsk_fs_attr_add_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
                 else {
                     endrun->next = data_run_cur;
                     data_run_cur->len -= run_len;
+                    data_run_cur->offset =
+                        a_data_run_new->offset + a_data_run_new->len;
                 }
 
                 return 0;
@@ -519,10 +531,10 @@ tsk_fs_attr_add_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
      * 0 if there is no list
      */
 
-    /* this is an error condition.  
-     * it means that we are currently at a greater VCN than
-     * what we are inserting, but we never found the filler
-     * for where we were to insert
+    /* This is an error condition.  
+     * It means that we cycled through the existing runs,
+     * ended at a VCN that is larger than what we are adding,
+     * and never found a filler entry to insert it into... 
      */
     if ((data_run_prev)
         && (data_run_prev->offset + data_run_prev->len >
@@ -531,19 +543,18 @@ tsk_fs_attr_add_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
         /* MAYBE this is because of a duplicate entry .. */
         if ((data_run_prev->addr == a_data_run_new->addr) &&
             (data_run_prev->len == a_data_run_new->len)) {
-            // @@@ Sould be we freeing this....?  What if the caller tries to write ti it?
+            // @@@ Sould be we freeing this....?  What if the caller tries to write to it?
             tsk_fs_attr_run_free(a_data_run_new);
             return 0;
         }
 
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_ARG;
+        tsk_errno = TSK_ERR_FS_GENFS;
         snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "fs_attr_run: error adding aditional run: %" PRIuDADDR
-            ", Previous %" PRIuDADDR " -> %" PRIuDADDR "   Current %"
-            PRIuDADDR " -> %" PRIuDADDR "\n", a_data_run_new->offset,
-            data_run_prev->addr, data_run_prev->len, a_data_run_new->addr,
-            a_data_run_new->len);
+            "fs_attr_add_run: error adding additional run (%" PRIuINUM
+            "): No filler entry for %" PRIuDADDR ". Final: %" PRIuDADDR,
+            a_fs_attr->fs_file->meta->addr, a_data_run_new->offset,
+            data_run_prev->offset + data_run_prev->len);
         return 1;
     }
 
@@ -574,18 +585,8 @@ tsk_fs_attr_add_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
         tmprun->len = a_data_run_new->offset - tmprun->offset;
         tmprun->flags = TSK_FS_ATTR_RUN_FLAG_FILLER;
         tmprun->next = a_data_run_new;
-
-        /* Adjust the length of the TSK_FS_ATTR structure to reflect the 
-         * new FILLER run
-         */
-        a_fs_attr->nrd.initsize += (tmprun->len * a_fs->block_size);
     }
 
-    /* Adjust the length of the TSK_FS_ATTR structure to reflect the 
-     * new run
-     */
-    a_fs_attr->nrd.initsize += (run_len * a_fs->block_size);
-
     // update the pointer to the end of the list
     a_fs_attr->nrd.run_end = a_data_run_new;
     while (a_fs_attr->nrd.run_end->next)
@@ -634,13 +635,9 @@ tsk_fs_attr_append_run(TSK_FS_INFO * a_fs, TSK_FS_ATTR * a_fs_attr,
 
     // update the rest of the offsets in the run (if any exist)
     data_run_cur = a_data_run;
-    a_fs_attr->nrd.initsize += (data_run_cur->len * a_fs->block_size);
     while (data_run_cur->next) {
         data_run_cur->next->offset =
             data_run_cur->offset + data_run_cur->len;
-
-        a_fs_attr->nrd.initsize +=
-            (data_run_cur->next->len * a_fs->block_size);
         a_fs_attr->nrd.run_end = data_run_cur->next;
         data_run_cur = data_run_cur->next;
     }
@@ -680,11 +677,11 @@ tsk_fs_attr_walk_res(const TSK_FS_ATTR * fs_attr,
     }
 
     /* Allocate a buffer that is at most a block size in length */
-    if ((a_flags & TSK_FS_FILE_WALK_FLAG_AONLY) == 0) {
-        buf_len = (size_t) fs_attr->size;
-        if (buf_len > fs->block_size)
-            buf_len = fs->block_size;
+    buf_len = (size_t) fs_attr->size;
+    if (buf_len > fs->block_size)
+        buf_len = fs->block_size;
 
+    if ((a_flags & TSK_FS_FILE_WALK_FLAG_AONLY) == 0) {
         if ((buf = tsk_malloc(buf_len)) == NULL) {
             return 1;
         }
@@ -820,7 +817,8 @@ tsk_fs_attr_walk_nonres(const TSK_FS_ATTR * fs_attr,
                 }
 
                 // we return 0s for reads past the initsize
-                else if (off > fs_attr->nrd.initsize) {
+                else if ((off >= fs_attr->nrd.initsize)
+                    && ((a_flags & TSK_FS_FILE_READ_FLAG_SLACK) == 0)) {
                     memset(buf, 0, fs->block_size);
                 }
                 else {
@@ -838,6 +836,12 @@ tsk_fs_attr_walk_nonres(const TSK_FS_ATTR * fs_attr,
                             PRIuDADDR, addr + len_idx);
                         return 1;
                     }
+                    if ((off + fs->block_size > fs_attr->nrd.initsize)
+                        && ((a_flags & TSK_FS_FILE_READ_FLAG_SLACK) == 0)) {
+                        memset(&buf[fs_attr->nrd.initsize - off], 0,
+                            fs->block_size -
+                            (size_t) (fs_attr->nrd.initsize - off));
+                    }
                 }
             }
 
@@ -987,7 +991,7 @@ tsk_fs_attr_walk(const TSK_FS_ATTR * a_fs_attr,
  * @param a_buf The buffer to read the data into.
  * @param a_len The number of bytes to read from the file.
  * @param a_flags Flags to use while reading
- * @returns The number of bytes read or -1 on error.
+ * @returns The number of bytes read or -1 on error (incl if offset is past end of file).
  */
 ssize_t
 tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
@@ -1017,107 +1021,98 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
 
     /* For resident data, copy data from the local buffer */
     else if (a_fs_attr->flags & TSK_FS_ATTR_RES) {
-        size_t read_len;
+        size_t len_toread;
 
-        if (a_offset > a_fs_attr->rd.buf_size) {
-            return 0;
+        if (a_offset >= a_fs_attr->size) {
+            tsk_error_reset();
+            tsk_errno = TSK_ERR_FS_READ_OFF;
+            snprintf(tsk_errstr, TSK_ERRSTR_L,
+                "tsk_fs_attr_read - %" PRIuOFF, a_offset);
+            return -1;
         }
 
-        if (a_len + a_offset > a_fs_attr->rd.buf_size)
-            read_len = a_fs_attr->rd.buf_size - (size_t) a_offset;
-        else
-            read_len = a_len;
+        len_toread = a_len;
+        if (a_offset + a_len > a_fs_attr->size) {
+            len_toread = (size_t) (a_fs_attr->size - a_offset);
+            memset(&a_buf[len_toread], 0, a_len - len_toread);
+        }
 
-        memcpy(a_buf, &a_fs_attr->rd.buf[a_offset], read_len);
-        return (ssize_t) read_len;
+        memcpy(a_buf, &a_fs_attr->rd.buf[a_offset], len_toread);
+
+        return (ssize_t) len_toread;
     }
 
     /* For non-resident data, load the needed block and copy the data */
     else if (a_fs_attr->flags & TSK_FS_ATTR_NONRES) {
         TSK_FS_ATTR_RUN *data_run_cur;
-        TSK_DADDR_T blkoffset;  // block offset of where we want to start reading from
-        size_t byteoffset;      // byte offset in blkoffset of where we want to start reading from
-        size_t size_remain;     // length remaining to copy
-        size_t size_tocopy;     // length total to copy
+        TSK_DADDR_T blkoffset_toread;   // block offset of where we want to start reading from
+        size_t byteoffset_toread;       // byte offset in blkoffset_toread of where we want to start reading from
+        size_t len_remain;      // length remaining to copy
+        size_t len_toread;      // length total to copy
 
-        if (a_offset > a_fs_attr->nrd.allocsize) {
-            return 0;
+        if (a_offset >= a_fs_attr->nrd.allocsize) {
+            tsk_error_reset();
+            tsk_errno = TSK_ERR_FS_READ_OFF;
+            snprintf(tsk_errstr, TSK_ERRSTR_L,
+                "tsk_fs_attr_read - %" PRIuOFF, a_offset);
+            return -1;
         }
 
-        // we return 0s for reads past the initsize
-        if (a_offset >= a_fs_attr->nrd.initsize) {
-            ssize_t len;
-
-            if (tsk_verbose)
-                fprintf(stderr,
-                    "tsk_fs_attr_read: Returning 0s for read past end of initsize (%"
-                    PRIuINUM ")\n", ((a_fs_attr->fs_file)
-                        && (a_fs_attr->fs_file->meta)) ? a_fs_attr->
-                    fs_file->meta->addr : 0);
-
-            if (a_offset + a_len > a_fs_attr->nrd.allocsize)
-                len = (ssize_t) (a_fs_attr->nrd.allocsize - a_offset);
-            else
-                len = (ssize_t) a_len;
-            memset(a_buf, 0, a_len);
-            return len;
-        }
-
-        blkoffset = a_offset / fs->block_size;
-        byteoffset = (size_t) (a_offset % fs->block_size);
+        blkoffset_toread = a_offset / fs->block_size;
+        byteoffset_toread = (size_t) (a_offset % fs->block_size);
 
         // determine how many bytes we can copy
+        len_toread = a_len;
         if (a_flags & TSK_FS_FILE_READ_FLAG_SLACK) {
             if (a_offset + a_len > a_fs_attr->nrd.allocsize)
-                size_tocopy =
+                len_toread =
                     (size_t) (a_fs_attr->nrd.allocsize - a_offset);
-            else
-                size_tocopy = a_len;
         }
         else {
             if (a_offset + a_len > a_fs_attr->size)
-                size_tocopy = (size_t) (a_fs_attr->size - a_offset);
-            else
-                size_tocopy = a_len;
+                len_toread = (size_t) (a_fs_attr->size - a_offset);
         }
-        size_remain = size_tocopy;
+        // wipe the buffer we won't read into
+        if (len_toread < a_len)
+            memset(&a_buf[len_toread], 0, a_len - len_toread);
+
+        len_remain = len_toread;
 
         // cycle through the run until we find where we can start to process the clusters
         for (data_run_cur = a_fs_attr->nrd.run; data_run_cur;
             data_run_cur = data_run_cur->next) {
-            TSK_DADDR_T run_offset;
-            size_t run_len;
+            TSK_DADDR_T blkoffset_inrun;
+            size_t len_inrun;
 
-            if (size_remain <= 0)
+            // we are done
+            if (len_remain <= 0)
                 break;
 
             // See if this run contains the starting offset they requested
-            if (data_run_cur->offset + data_run_cur->len < blkoffset)
+            if (data_run_cur->offset + data_run_cur->len <=
+                blkoffset_toread)
                 continue;
 
             // block offset into this run
-            run_offset = blkoffset - data_run_cur->offset;
+            blkoffset_inrun = blkoffset_toread - data_run_cur->offset;
 
             // see if we need to read the rest of this run and into the next or if it is all here
-            if (fs->block_size * (data_run_cur->len - run_offset) >=
-                size_remain) {
-                run_len = size_remain;
-            }
-            else {
-                run_len =
-                    (size_t) (fs->block_size * (data_run_cur->len -
-                        run_offset));
-            }
+            len_inrun = len_remain;
+            if ((data_run_cur->len - blkoffset_inrun) * fs->block_size <
+                len_remain)
+                len_inrun =
+                    (size_t) ((data_run_cur->len -
+                        blkoffset_inrun) * fs->block_size);
 
-            /* sparse files just get 0s */
+            /* sparse files/runs just get 0s */
             if (data_run_cur->flags & TSK_FS_ATTR_RUN_FLAG_SPARSE) {
-                memset(&a_buf[size_tocopy - size_remain], 0, run_len);
+                memset(&a_buf[len_toread - len_remain], 0, len_inrun);
             }
             /* FILLER entries exist when the source file system can store run
              * info out of order and we did not get all of the run info.  We
              * return 0s if data is read from this type of run. */
             else if (data_run_cur->flags & TSK_FS_ATTR_RUN_FLAG_FILLER) {
-                memset(&a_buf[size_tocopy - size_remain], 0, run_len);
+                memset(&a_buf[len_toread - len_remain], 0, len_inrun);
                 if (tsk_verbose)
                     fprintf(stderr,
                         "tsk_fs_attr_read_type: File %" PRIuINUM
@@ -1125,38 +1120,67 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
                         (a_fs_attr->fs_file->meta) ? a_fs_attr->fs_file->
                         meta->addr : 0);
             }
+            // we return 0s for reads past the initsize (unless they want slack space)
+            else if (((TSK_OFF_T) ((data_run_cur->offset +
+                            blkoffset_inrun) * fs->block_size +
+                        byteoffset_toread) >= a_fs_attr->nrd.initsize)
+                && ((a_flags & TSK_FS_FILE_READ_FLAG_SLACK) == 0)) {
+                memset(&a_buf[len_toread - len_remain], 0, len_inrun);
+                if (tsk_verbose)
+                    fprintf(stderr,
+                        "tsk_fs_attr_read: Returning 0s for read past end of initsize (%"
+                        PRIuINUM ")\n", ((a_fs_attr->fs_file)
+                            && (a_fs_attr->fs_file->meta)) ? a_fs_attr->
+                        fs_file->meta->addr : 0);
+            }
             else {
                 TSK_OFF_T fs_offset_b;
                 ssize_t cnt;
 
                 // calcuate the byte offset in the file system
                 fs_offset_b =
-                    (data_run_cur->addr + run_offset) * fs->block_size;
+                    (data_run_cur->addr +
+                    blkoffset_inrun) * fs->block_size;
 
                 // add the byte offset in the block
-                fs_offset_b += byteoffset;
+                fs_offset_b += byteoffset_toread;
 
                 // reset this in case we need to also read from the next run 
-                byteoffset = 0;
+                byteoffset_toread = 0;
 
                 cnt =
                     tsk_fs_read(fs, fs_offset_b,
-                    &a_buf[size_tocopy - size_remain], run_len);
-                if (cnt != run_len) {
+                    &a_buf[len_toread - len_remain], len_inrun);
+                if (cnt != len_inrun) {
                     if (cnt >= 0) {
                         tsk_error_reset();
                         tsk_errno = TSK_ERR_FS_READ;
                     }
                     snprintf(tsk_errstr2, TSK_ERRSTR_L,
                         "tsk_fs_attr_read_type: offset: %" PRIuOFF
-                        "  Len: %" PRIuSIZE "", fs_offset_b, run_len);
+                        "  Len: %" PRIuSIZE "", fs_offset_b, len_inrun);
                     return cnt;
                 }
-            }
-            size_remain -= run_len;
-        }
 
-        return (ssize_t) (size_tocopy - size_remain);
+                // see if part of the data is in the non-initialized space
+                if (((TSK_OFF_T) ((data_run_cur->offset +
+                                blkoffset_inrun) * fs->block_size +
+                            byteoffset_toread + len_inrun) >
+                        a_fs_attr->nrd.initsize)
+                    && ((a_flags & TSK_FS_FILE_READ_FLAG_SLACK) == 0)) {
+                    size_t off =
+                        (size_t) ((data_run_cur->offset +
+                            blkoffset_inrun) * fs->block_size +
+                        byteoffset_toread + len_inrun -
+                        a_fs_attr->nrd.initsize);
+                    memset(&a_buf[len_toread - len_remain + off], 0,
+                        len_inrun - off);
+                }
+
+            }
+            len_remain -= len_inrun;
+        }
+        return (ssize_t) (len_toread - len_remain);
     }
 
     tsk_errno = TSK_ERR_FS_ARG;
