@@ -26,6 +26,7 @@
  */
 
 #include "tsk_fs_i.h"
+#include "tsk_hfs.h"
 
 
 /*******************************************************************************
@@ -250,23 +251,15 @@ tsk_fs_path2inum(TSK_FS_INFO * a_fs, const char *a_path,
              * Check if this is the name that we are currently looking for,
              * as identified in 'cur_dir'
              */
-            if (TSK_FS_TYPE_ISFFS(a_fs->ftype)
-                || TSK_FS_TYPE_ISEXT(a_fs->ftype)) {
+            /* FAT is a special case because we check the short name */
+            if (TSK_FS_TYPE_ISFAT(a_fs->ftype)) {
                 if ((fs_file->name->name)
-                    && (strcmp(fs_file->name->name, cur_dir) == 0)) {
-                    found_name = 1;
-                }
-            }
-            /* FAT is a special case because we do case insensitive and we check
-             * the short name 
-             */
-            else if (TSK_FS_TYPE_ISFAT(a_fs->ftype)) {
-                if ((fs_file->name->name)
-                    && (strcasecmp(fs_file->name->name, cur_dir) == 0)) {
+                    && (a_fs->name_cmp(a_fs, fs_file->name->name,
+                            cur_dir) == 0)) {
                     found_name = 1;
                 }
                 else if ((fs_file->name->shrt_name)
-                    && (strcasecmp(fs_file->name->shrt_name,
+                    && (a_fs->name_cmp(a_fs, fs_file->name->shrt_name,
                             cur_dir) == 0)) {
                     found_name = 1;
                 }
@@ -275,7 +268,8 @@ tsk_fs_path2inum(TSK_FS_INFO * a_fs, const char *a_path,
             /* NTFS gets a case insensitive comparison */
             else if (TSK_FS_TYPE_ISNTFS(a_fs->ftype)) {
                 if ((fs_file->name->name)
-                    && (strcasecmp(fs_file->name->name, cur_dir) == 0)) {
+                    && (a_fs->name_cmp(a_fs, fs_file->name->name,
+                            cur_dir) == 0)) {
                     /*  ensure we have the right attribute name */
                     if (cur_attr == NULL) {
                         found_name = 1;
@@ -293,7 +287,7 @@ tsk_fs_path2inum(TSK_FS_INFO * a_fs, const char *a_path,
                                     continue;
 
                                 if ((fs_attr->name)
-                                    && (strcasecmp(fs_attr->name,
+                                    && (a_fs->name_cmp(a_fs, fs_attr->name,
                                             cur_attr) == 0)) {
                                     found_name = 1;
                                 }
@@ -302,16 +296,12 @@ tsk_fs_path2inum(TSK_FS_INFO * a_fs, const char *a_path,
                     }
                 }
             }
-
-            /* Unknown how to compare names in this filesystem */
             else {
-                tsk_fs_dir_close(fs_dir);
-                free(cpath);
-                tsk_errno = TSK_ERR_FS_GENFS;
-                snprintf(tsk_errstr, TSK_ERRSTR_L,
-                    "tsk_fs_path2inum: File System type not supported for file name comparison (%X)",
-                    a_fs->ftype);
-                return -1;
+                if ((fs_file->name->name)
+                    && (a_fs->name_cmp(a_fs, fs_file->name->name,
+                            cur_dir) == 0)) {
+                    found_name = 1;
+                }
             }
 
             if (found_name) {
@@ -437,7 +427,7 @@ tsk_fs_ifind_path(TSK_FS_INFO * fs, TSK_TCHAR * tpath, TSK_INUM_T * result)
         ptr16 = (UTF16 *) tpath;
 
         retval =
-            tsk_UTF16toUTF8(fs->endian, (const UTF16 **) &ptr16, (UTF16 *)
+            tsk_UTF16toUTF8_lclorder((const UTF16 **) &ptr16, (UTF16 *)
             & ptr16[TSTRLEN(tpath) + 1], &ptr8,
             (UTF8 *) ((uintptr_t) ptr8 + clen), TSKlenientConversion);
         if (retval != TSKconversionOK) {
@@ -562,13 +552,12 @@ ifind_data_act(TSK_FS_FILE * fs_file, void *ptr)
             data->curtype = fs_attr->type;
             data->curid = fs_attr->id;
             if (fs_attr->flags & TSK_FS_ATTR_NONRES) {
-                if (tsk_fs_file_walk_type(fs_file, fs_attr->type,
-                        fs_attr->id, file_flags, ifind_data_file_ntfs_act,
-                        ptr)) {
+                if (tsk_fs_attr_walk(fs_attr,
+                        file_flags, ifind_data_file_ntfs_act, ptr)) {
                     if (tsk_verbose)
                         tsk_fprintf(stderr,
-                            "Error walking file %" PRIuINUM,
-                            fs_file->meta->addr);
+                            "Error walking file %" PRIuINUM
+                            " Attribute: %i", fs_file->meta->addr, i);
 
                     /* Ignore these errors */
                     tsk_error_reset();
@@ -594,7 +583,8 @@ ifind_data_act(TSK_FS_FILE * fs_file, void *ptr)
      * correlated with the incorrect inode
      */
     else {
-        // @@@ Need to add handling back in here to find indirect blocks (once a soln is found)
+        const TSK_FS_ATTR *fs_attr;
+
         if (tsk_fs_file_walk(fs_file, file_flags,
                 ifind_data_file_act, ptr)) {
             if (tsk_verbose)
@@ -604,6 +594,25 @@ ifind_data_act(TSK_FS_FILE * fs_file, void *ptr)
             /* Ignore these errors */
             tsk_error_reset();
         }
+
+
+        // try the indirect blocks
+        fs_attr = tsk_fs_file_attr_get_type(fs_file,
+            TSK_FS_ATTR_TYPE_UNIX_INDIR, 0, 0);
+        if (fs_attr) {
+            data->curtype = fs_attr->type;
+            data->curid = fs_attr->id;
+
+            if (tsk_fs_attr_walk(fs_attr,
+                    file_flags, ifind_data_file_act, ptr)) {
+                if (tsk_verbose)
+                    tsk_fprintf(stderr,
+                        "Error walking file %" PRIuINUM
+                        " Indirect Attribute", fs_file->meta->addr);
+                /* Ignore these errors */
+                tsk_error_reset();
+            }
+        }
     }
 
     return TSK_WALK_CONT;
