@@ -456,8 +456,8 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                         tsk_UTF16toUTF8(fs->endian,
                         (const UTF16 **) &name16,
                         (UTF16 *) & buf[b_offs + dentry->fi_len], &name8,
-                        (UTF8 *) ((uintptr_t) & in_node->inode.
-                            fn[ISO9660_MAXNAMLEN_STD]),
+                        (UTF8 *) ((uintptr_t) & in_node->
+                            inode.fn[ISO9660_MAXNAMLEN_STD]),
                         TSKlenientConversion);
                     if (retVal != TSKconversionOK) {
                         if (tsk_verbose)
@@ -928,6 +928,8 @@ iso9660_dinode_copy(ISO_INFO * iso, TSK_FS_META * fs_meta)
 
     fs_meta->crtime = mktime(&t);
     fs_meta->mtime = fs_meta->atime = fs_meta->ctime = 0;
+    fs_meta->crtime_nano = fs_meta->mtime_nano = fs_meta->atime_nano =
+        fs_meta->ctime_nano = 0;
 
     if (iso->dinode->dr.flags & ISO9660_FLAG_DIR)
         fs_meta->type = TSK_FS_META_TYPE_DIR;
@@ -1302,7 +1304,7 @@ iso9660_make_data_run(TSK_FS_FILE * a_fs_file)
     // initialize the data run
     if (tsk_fs_attr_set_run(a_fs_file, fs_attr, data_run, NULL,
             TSK_FS_ATTR_TYPE_DEFAULT, TSK_FS_ATTR_ID_DEFAULT,
-            a_fs_file->meta->size,
+            a_fs_file->meta->size, a_fs_file->meta->size,
             roundup(a_fs_file->meta->size + dd.ext_len,
                 fs->block_size) - dd.ext_len, 0, 0)) {
         return 1;
@@ -1965,7 +1967,7 @@ load_vol_desc(TSK_FS_INFO * fs)
     iso->pvd = NULL;
     iso->svd = NULL;
     fs->block_size = 0;
-    fs->dev_bsize = 512;
+    fs->dev_bsize = fs->img_info->sector_size;
 
 #if 0
     b = (iso_bootrec *) tsk_malloc(sizeof(iso_bootrec));
@@ -2267,6 +2269,7 @@ iso9660_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
     fs->fscheck = iso9660_fscheck;
     fs->istat = iso9660_istat;
     fs->close = iso9660_close;
+    fs->name_cmp = iso9660_name_cmp;
 
     fs->jblk_walk = iso9660_jblk_walk;
     fs->jentry_walk = iso9660_jentry_walk;
